/*
 * Copyright (c) 1996, 2003 VIA Networking Technologies, Inc.
 * All rights reserved.
 *
 * This software is copyrighted by and is the sole property of
 * VIA Networking Technologies, Inc. This software may only be used
 * in accordance with the corresponding license agreement. Any unauthorized
 * use, duplication, transmission, distribution, or disclosure of this
 * software is expressly forbidden.
 *
 * This software is provided by VIA Networking Technologies, Inc. "as is"
 * and any express or implied warranties, including, but not limited to, the
 * implied warranties of merchantability and fitness for a particular purpose
 * are disclaimed. In no event shall VIA Networking Technologies, Inc.
 * be liable for any direct, indirect, incidental, special, exemplary, or
 * consequential damages.   
 *
 *
 * File: wetctl.c
 *
 * Purpose: 
 *
 *      Command mode utility for Solomon 802.11a/b/g wireless card
 *
 * Author: lyndon chen
 *
 * Date: Jan 2, 2003
 *
 * Functions:
 *      s_cmdBSSJoin- join an existing BSS set via ioctl command
 *      s_cmdScan- scan all channels via ioctl command
 *      s_cmdSetWEP- set wep key via ioctl command  
 *      s_cmdGetLink- get link status via ioctl command
 *      s_cmdGetList- get scan list via ioctl command
 *      s_cmdGetMIB- get DOT11 MIB via ioctl command
 *      s_cmdGetSTAT- get Statistics via ioctl command
 *      vShowList- print scan list
 *      vByteToWEP232- byte string to Hex key
 *      vAddrToStr- print MAC address
 *      vCmdUsage- command help list
 * Revision History:
 *
 */
 
 
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include "wetctl.h"
#include "ttype.h"
#include "iocmd.h"

/*---------------------  Static Definitions -------------------------*/

/*---------------------  Static Classes  ----------------------------*/

/*---------------------  Static Variables  --------------------------*/

BYTE AppName[20];
/*---------------------  Static Functions  --------------------------*/


static int s_cmdBSSJoin(int skfd, int argc, char **argv);
static int s_cmdAPStart(int skfd, int argc, char **argv);
static int s_cmdScan(int skfd, int argc, char **argv);
static int s_cmdSetWEP(int skfd, int argc, char **argv);
static int s_cmdGetLink(int skfd, int argc, char **argv);
static int s_cmdGetList(int skfd, int argc, char **argv);
static int s_cmdGetMIB(int skfd, int argc, char **argv);
//static int s_cmdGetSTAT(int skfd, int argc, char **argv);
static int s_cmdStopMAC(int skfd, int argc, char **argv);
static int s_cmdStartMAC(int skfd, int argc, char **argv);
static int s_cmdGetNodeList(int skfd, int argc, char **argv);
static VOID vShowList(PSBSSIDList pList);
static VOID vShowNodeList(PSNodeList pList);
static VOID vByteToWEP232(PBYTE abyKey, char *abyStr);
static VOID vAddrToStr(PBYTE abyAddr, PBYTE abyStr);
static VOID vWEPToStr(PBYTE abyWEP, PBYTE abyStr);
static VOID vCmdUsage(void);

/*---------------------  Export Variables  --------------------------*/

/*---------------------  Export Functions  --------------------------*/

/*+
 *
 * Description:
 *    Entry of utility 
 *
 * Parameters:
 *  In:
 *      argc, argv
 *  Out:
 *      none
 *
 * Return Value: int
 *
-*/

int main(int argc, char *argv[]) {
    int skfd;
	int result = 0;
	SCmdRequest	sReq;
	
	strcpy((char*)AppName, APPNAME);
	
	if (argc <= 2) {
	    vCmdUsage();
	    result = 1;
	    return result;
	};
	    
        
	// get a socket
	skfd = socket(AF_INET, SOCK_STREAM, 0);
	
	if (skfd < 0) {
	    printf("%s: can't find the socket driver!\n", AppName);
	    result = 1;
	    return result;
	};   
	    
	strcpy((char *)sReq.name, argv[1]);
	sReq.wResult = 0;
	sReq.data = NULL;
	sReq.wCmdCode = 0;
	result = ioctl(skfd, IOCTL_CMD_TEST, &sReq);
	
	if ((result < 0) || (sReq.wResult != MAGIC_CODE)) {
		printf("%s: failed in testing interface, '%s'may be not running.",
				AppName, argv[1]);
        				
	    result = 1;
	    return result;				
	}

	if (strcasecmp(argv[2], PRIVACY) == 0) {
	    result = s_cmdSetWEP(skfd, argc, argv);
	}
	else if (strcasecmp(argv[2], SCAN) == 0) {
	    result = s_cmdScan(skfd, argc, argv);
	}
	else if (strcasecmp(argv[2], JOIN) == 0) {
	    result = s_cmdBSSJoin(skfd, argc, argv);
	}
	else if ( strcasecmp(argv[2], GETLIST) == 0) {
	    result = s_cmdGetList(skfd, argc, argv);
	}
	else if ( strcasecmp(argv[2], GETLINK) == 0) {
	    result = s_cmdGetLink(skfd, argc, argv);
	}
	else if ( strcasecmp(argv[2], GETMIB) == 0) {
	    result = s_cmdGetMIB(skfd, argc, argv);
	}
/*	
	else if ( strcasecmp(argv[2], GETSTAT) == 0) {
	    result = s_cmdGetSTAT(skfd, argc, argv);
	}	
*/	
	else if (strcasecmp(argv[2], STOP) == 0) {
	    result = s_cmdStopMAC(skfd, argc, argv);
	}
	else if (strcasecmp(argv[2], START) == 0) {
	    result = s_cmdStartMAC(skfd, argc, argv);
	}
	else if (strcasecmp(argv[2], APSTART) == 0) {
	    result = s_cmdAPStart(skfd, argc, argv);
	}
	else if (strcasecmp(argv[2], GETNODE) == 0) {
	    result = s_cmdGetNodeList(skfd, argc, argv);
	}	
	else {
		printf("%s: %s is an invalid command\n",AppName, argv[2]);
		vCmdUsage();
		result = 1;
	}
	
	if (result < 0) {
	    printf("%s: failed in ioctl sRequest! result= %d\n",
				AppName, result);
    }

	return result;
}

/*+
 *
 * Description:
 *     Reformat input string to wep key   
 *
 * Parameters:
 *  In:
 *      array of string
 *  Out:
 *      array of hex char
 *
 * Return Value: none
 *
-*/

VOID vByteToWEP232(PBYTE abyKey, char *abyStr) {
	PBYTE	pChr;
	int		ii;
	UINT	val;

    // i = 29 byte/232 bit
	for (ii = 0; ii < 29; ii++) {
		pChr = (PBYTE)strchr(abyStr, ':');
		if (pChr == NULL) {
            break;
		}
		else {
			*pChr = '\0';
			sscanf((char *)abyStr, "%x", &val);
			abyKey[ii] = (BYTE)val;
			abyStr = (char *)(pChr+1);
		}
	}
    sscanf((char *)abyStr, "%x", &val);
    abyKey[ii] = (BYTE)val;
}



/*+
 *
 * Description:
 *     Print MAC address
 *
 * Parameters:
 *  In:
 *      MAC address
 *  Out:
 *      none
 *
 * Return Value: none
 *
-*/

VOID vAddrToStr(PBYTE abyAddr, PBYTE abyStr)
{
	sprintf((char *)abyStr, "%02x:%02x:%02x:%02x:%02x:%02x", 
		abyAddr[0], abyAddr[1], abyAddr[2], abyAddr[3], abyAddr[4], abyAddr[5]);
}


/*+
 *
 * Description:
 *     Print WEP key
 *
 * Parameters:
 *  In:
 *      array of hex wep key
 *  Out:
 *      array of wep key string
 *
 * Return Value: none
 *
-*/

VOID vWEPToStr(PBYTE abyWEP, PBYTE abyStr)
{
	sprintf((char *)abyStr, "%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x", 
		abyWEP[0], abyWEP[1], abyWEP[2], abyWEP[3], abyWEP[4], abyWEP[5],
		abyWEP[6], abyWEP[7], abyWEP[8], abyWEP[9], abyWEP[10], abyWEP[11],
		abyWEP[12]		
		);
}


/*+
 *
 * Description:
 *
 *  Called when the user set the 'scan' command at the
 *	command line to scan all available channels. 
 *     
 *
 * Parameters:
 *  In:
 *      skfd, argc, argv
 *  Out:
 *      none
 *
 * Return Value: '0' if success
 *
-*/


int s_cmdScan(int skfd, int argc, char **argv) {
	int			result = 0;
	SCmdRequest sReq;
	SCmdScan	cmd;
	
	strcpy((char *)sReq.name, argv[1]);
	cmd.ssid[0] = SSID_ID;
	cmd.ssid[1] = 0;
	cmd.ssid[2] = 0;
	if (argc > 3) {
		if ((cmd.ssid[1] = strlen(argv[3])) <= 32 ) {
		    if (strcmp(argv[3], "any") != 0 ) {
			    memcpy( &(cmd.ssid[2]), argv[3], cmd.ssid[1]);
			}    
		}	    
	}
	sReq.wResult = 0;
	sReq.data = &cmd;
    sReq.wCmdCode = WLAN_CMD_BSS_SCAN;
	result = ioctl(skfd, IOCTL_CMD_SET, &sReq);
	if (sReq.wResult) {
		printf( "%s: fail in scan command !\n",AppName);
		result = 1;
	}

    return result;
}


/*+
 *
 * Description:
 *
 *  Called when the user set the 'stop' command at the
 *	command line to stop the MAC 
 *
 * Parameters:
 *  In:
 *      skfd, argc, argv
 *  Out:
 *      none
 *
 * Return Value: '0' if success
 *
-*/

int s_cmdStopMAC( int skfd, int argc, char **argv ) {
	int			result = 0;
	SCmdRequest	sReq;
	

	strcpy((char *)sReq.name, argv[1]);
	sReq.wResult = 0;
	sReq.data = NULL;
    sReq.wCmdCode = WLAN_CMD_STOP_MAC;
	result = ioctl(skfd, IOCTL_CMD_SET, &sReq);
	if ( sReq.wResult ) {
		printf( "%s: fail in stop command !\n",AppName);
		result = 1;
	}

    return result;
}


/*+
 *
 * Description:
 *
 *	Called when the user set the 'start' command at the
 *	command line to enable the MAC 
 *
 * Parameters:
 *  In:
 *      skfd, argc, argv
 *  Out:
 *      none
 *
 * Return Value: '0' if success
 *
-*/

int s_cmdStartMAC( int skfd, int argc, char **argv ) {
	int			result = 0;
	SCmdRequest	sReq;
	

	strcpy((char *)sReq.name, argv[1]);
	sReq.wResult = 0;
	sReq.data = NULL;
    sReq.wCmdCode = WLAN_CMD_START_MAC;
	result = ioctl(skfd, IOCTL_CMD_SET, &sReq);
	if ( sReq.wResult ) {
		printf( "%s: fail in start command !\n",AppName);
		result = 1;
	}

    return result;
}


/*+
 *
 * Description:
 *
 *	Called when the user set 'join' command at the
 *	command line to join an existing BSS set
 *
 * Parameters:
 *  In:
 *      skfd, argc, argv
 *  Out:
 *      none
 *
 * Return Value: '0' if success
 *
-*/

int s_cmdBSSJoin(int skfd, int argc, char **argv) {
	int					result = 0;
	SCmdRequest	        sReq;
	SCmdBSSJoin		    cmd;

	if ( argc < 5 ) {
		printf("%s: not enough arguments for the 'join' command;\n ", AppName);
		vCmdUsage();
		result = 1;
	}
	else {
		strcpy((char *)sReq.name, argv[1]);
		sReq.wResult = 0;
		sReq.data = &cmd;
		sReq.wCmdCode = WLAN_CMD_BSS_JOIN;

		if ( (strcmp(argv[3], "adhoc") == 0) ||
		     (strcmp(argv[3], "ad-hoc") == 0) ) {
			cmd.wBSSType = ADHOC;
		}
		else {
			cmd.wBSSType = INFRA;
		}		
		cmd.ssid[0] = SSID_ID;
		if ((cmd.ssid[1] = strlen(argv[4])) <= 32 ) {
		    if (strcmp(argv[4], "any") == 0 ) {
			    cmd.ssid[1] = 0;
			    cmd.ssid[2] = 0;
            }
            else { 
			    memcpy( &(cmd.ssid[2]), argv[4], cmd.ssid[1]);
			}    
		}
		else {
			printf("warning: ssid longer than 32 chars set to 'any ssid'");
			cmd.ssid[1] = 0;
			cmd.ssid[2] = 0;
		}
		
		cmd.uChannel = atoi(argv[5]);
		if (strcmp(argv[6], "yes") == 0) {
			cmd.bPSEnable = TRUE;
		}
		else {
			cmd.bPSEnable= FALSE;
		}		
		
		if (strcmp(argv[7], "share") == 0) {
			cmd.bShareKeyAuth = TRUE;
		}
		else {
			cmd.bShareKeyAuth = FALSE;		    
		}
						
		result = ioctl( skfd, IOCTL_CMD_SET, &sReq);
		
		if ( sReq.wResult ) {
			printf("%s: Fail in Join a wireless network command !\n", AppName);
			result = 1;
		}
	}

	return result;
}



/*+
 *
 * Description:
 *
 *	Called when set 'apstart' command at the
 *	command line to start ESS
 *
 * Parameters:
 *  In:
 *      skfd, argc, argv
 *  Out:
 *      none
 *
 * Return Value: '0' if success
 *
-*/


int s_cmdAPStart(int skfd, int argc, char **argv) {
	int					result = 0;
	SCmdRequest	        sReq;
	SCmdStartAP   	    cmd;


	if ( argc < 6 ) {
		printf("%s: not enough arguments for the 'startap' command;\n ", AppName);
		vCmdUsage();
		result = 1;
	}
	else {
		strcpy((char *)sReq.name, argv[1]);
		sReq.wResult = 0;
		sReq.data = &cmd;
		sReq.wCmdCode = WLAN_CMD_AP_START;

		if (strcmp(argv[3], "ap") == 0) {
			cmd.wBSSType = AP;
		}
		else {
			cmd.wBSSType =INFRA;
		}		
		cmd.ssid[0] = SSID_ID;
		if ((cmd.ssid[1] = strlen(argv[4])) <= 32 ) {
		    if (strcmp(argv[4], "any") == 0 ) {
			    cmd.ssid[1] = 0;
			    cmd.ssid[2] = 0;
            }
            else { 
			    memcpy( &(cmd.ssid[2]), argv[4], cmd.ssid[1]);
			}    
		}
		else {
			printf("warning: ssid longer than 32 chars set to 'any ssid'");
			cmd.ssid[1] = 0;
			cmd.ssid[2] = 0;
		}
		
		cmd.uChannel = atoi(argv[5]);	
		cmd.uBeaconInt = atoi(argv[6]);
			
		if (strcmp(argv[7], "share") == 0) {
			cmd.bShareKeyAuth = TRUE;
		}
		else {
			cmd.bShareKeyAuth = FALSE;		    
		}	
		cmd.byBasicRate = atoi(argv[8]);
		
		cmd.wBBPType = atoi(argv[9]);
		
		result = ioctl( skfd, IOCTL_CMD_SET, &sReq);
		
		if ( sReq.wResult ) {
			printf("%s: Fail in start a wireless network command !\n", AppName);
			result = 1;
		}
	}

	return result;
}



/*+
 *
 * Description:
 *
 *	Called when the user has requested the 'privacy' command at the
 *	command line to set WEP key. 
 *
 * Parameters:
 *  In:
 *      skfd, argc, argv
 *  Out:
 *      none
 *
 * Return Value: '0' if success
 *
-*/

int s_cmdSetWEP(int skfd, int argc, char **argv) {
	int					result, ii;
	SCmdRequest	        sReq;
	SCmdSetWEP		    cmd;
	int                 StrLen;

	result = 0;

    if ( argc < 4 ) {
    		printf("%s: not enough arguments for the privacy command;\n ", AppName);
    		vCmdUsage();
    		result = 1;
            return result;		
    }
    if (strcmp(argv[3], "off") == 0) {
        cmd.bEnableWep = FALSE;
    }
    else {
        cmd.bEnableWep = TRUE;            
    	if ( argc < 5 ) {
    		printf("%s: not enough arguments for the privacy command;\n ", AppName);
    		vCmdUsage();
    		result = 1;
            return result;		
    	}
    	else {
    	    memset(&cmd.abyWepKey, 0, (WEP_NKEYS * WEP_KEYMAXLEN)); 
    		cmd.byKeyIndex = atoi(argv[3]);
		    for(ii = 4; ii < argc ; ii++) {
                if (strcmp(argv[ii], "key0") == 0) {
                    if (strcmp(argv[ii+1], "no") != 0) {
		                StrLen = strlen(argv[ii+1]);
		                if (StrLen <= 14) {
		                    if (StrLen == 14) {
                                cmd.auWepKeyLength[0] = WEP_40BIT_LEN;
		                    }else {
		                         printf("%s: WEP_KEY0 40bit format error!\n", AppName);
		                         break;
		                    }     
		                }       //38 = 26 + 12
		                else if (StrLen <= 38) {
		                    if (StrLen == 38) {
                                cmd.auWepKeyLength[0] = WEP_104BIT_LEN;
		                    }else {
		                         printf("%s: WEP_KEY0 104bit format error!\n", AppName);
		                         break;
		                    }     		                    
		                }       //86 = 29x2 + 28
		                else if (StrLen <= 86) {
		                    if (StrLen == 86) {
                                cmd.auWepKeyLength[0] = WEP_232BIT_LEN;
		                    }else {
		                         printf("%s: WEP_KEY0 232bit format error!\n", AppName);
		                         break;
		                    }		                    
		                }
		                vByteToWEP232(&(cmd.abyWepKey[0][0]), argv[ii+1]);
		                cmd.bWepKeyAvailable[0] = TRUE;
		            }
		            else {
		                cmd.bWepKeyAvailable[0] = FALSE;
		            }    
                }
                
                if (strcmp(argv[ii], "key1") == 0) {
                    if (strcmp(argv[ii+1], "no") != 0) {
		                StrLen = strlen(argv[ii+1]);
		                if (StrLen <= 14) {
		                    if (StrLen == 14) {
                                cmd.auWepKeyLength[1] = WEP_40BIT_LEN;
		                    }else {
		                         printf("%s: WEP_KEY1 40bit format error!\n", AppName);
		                         break;
		                    }     
		                }       //38 = 26 + 12
		                else if (StrLen <= 38) {
		                    if (StrLen == 38) {
                                cmd.auWepKeyLength[1] = WEP_104BIT_LEN;
		                    }else {
		                         printf("%s: WEP_KEY1 104bit format error!\n", AppName);
		                         break;
		                    }     		                    
		                }       //47 = 32 + 15
		                else if (StrLen <= 86) {
		                    if (StrLen == 86) {
                                cmd.auWepKeyLength[1] = WEP_232BIT_LEN;
		                    }else {
		                         printf("%s: WEP_KEY1 232bit format error!\n", AppName);
		                         break;
		                    }		                    
		                }
		                vByteToWEP232(&(cmd.abyWepKey[1][0]), argv[ii+1]);
		                cmd.bWepKeyAvailable[1] = TRUE;
		            }
		            else {
		                cmd.bWepKeyAvailable[1] = FALSE;
		            }    
                }
                
                if (strcmp(argv[ii], "key2") == 0) {
                    if (strcmp(argv[ii+1], "no") != 0) {
		                StrLen = strlen(argv[ii+1]);
		                if (StrLen <= 14) {
		                    if (StrLen == 14) {
                                cmd.auWepKeyLength[2] = WEP_40BIT_LEN;
		                    }else {
		                         printf("%s: WEP_KEY2 40bit format error!\n", AppName);
		                         break;
		                    }     
		                }       //38 = 26 + 12
		                else if (StrLen <= 38) {
		                    if (StrLen == 38) {
                                cmd.auWepKeyLength[2] = WEP_104BIT_LEN;
		                    }else {
		                         printf("%s: WEP_KEY2 104bit format error!\n", AppName);
		                         break;
		                    }     		                    
		                }       //47 = 32 + 15
		                else if (StrLen <= 86) {
		                    if (StrLen == 86) {
                                cmd.auWepKeyLength[2] = WEP_232BIT_LEN;
		                    }else {
		                         printf("%s: WEP_KEY2 232bit format error!\n", AppName);
		                         break;
		                    }		                    
		                }
		                vByteToWEP232(&(cmd.abyWepKey[2][0]), argv[ii+1]);
		                cmd.bWepKeyAvailable[2] = TRUE;
		            }
		            else {
		                cmd.bWepKeyAvailable[2] = FALSE;
		            }    
                }
                  
                if (strcmp(argv[ii], "key3") == 0) {
                    if (strcmp(argv[ii+1], "no") != 0) {
		                StrLen = strlen(argv[ii+1]);
		                if (StrLen <= 14) {
		                    if (StrLen == 14) {
                                cmd.auWepKeyLength[3] = WEP_40BIT_LEN;
		                    }else {
		                         printf("%s: WEP_KEY3 40bit format error!\n", AppName);
		                         break;
		                    }     
		                }       //38 = 26 + 12
		                else if (StrLen <= 38) {
		                    if (StrLen == 38) {
                                cmd.auWepKeyLength[3] = WEP_104BIT_LEN;
		                    }else {
		                         printf("%s: WEP_KEY3 104bit format error!\n", AppName);
		                         break;
		                    }     		                    
		                }       //47 = 32 + 15
		                else if (StrLen <= 86) {
		                    if (StrLen == 86) {
                                cmd.auWepKeyLength[3] = WEP_232BIT_LEN;
		                    }else {
		                         printf("%s: WEP_KEY3 232bit format error!\n", AppName);
		                         break;
		                    }		                    
		                }
		                vByteToWEP232(&(cmd.abyWepKey[3][0]), argv[ii+1]);
		                cmd.bWepKeyAvailable[3] = TRUE;
		            }
		            else {
		                cmd.bWepKeyAvailable[3] = FALSE;
		            }    
                }                                              
		    }        		
    	}
	}
	strcpy((char *)sReq.name, argv[1]);
	sReq.wResult = 0;
	sReq.data = &cmd;
	sReq.wCmdCode = WLAN_CMD_SET_WEP;	
	result = ioctl( skfd, IOCTL_CMD_SET, &sReq);
	
	if (sReq.wResult) {
		printf("%s: Either the wep key was invalid or mac function didn't work.\n",
			    AppName);
		result = 1;
	}
		
	return result;
}



/*+
 *
 * Description:
 *
 *	Called when the user has requested the 'list' command at the
 *	command line.
 *	This function prints all the BSS's in the current list of BSS's
 *	generated from the last scan. 
 *
 * Parameters:
 *  In:
 *      skfd, argc, argv
 *  Out:
 *      none
 *
 * Return Value: '0' if success
 *
-*/

int s_cmdGetList(int skfd, int argc, char **argv)
{
	int					result;
	SCmdRequest	        sReq;
	SBSSIDList		    sList;
    PSBSSIDList         pList;

	result = 0;
    memset(&sList, 0, sizeof(SBSSIDList));	
	strcpy((char *)sReq.name, argv[1]);
	sReq.wResult = 0;
	sReq.data = &sList;
	sReq.wCmdCode = WLAN_CMD_GET_LISTLEN;
	result = ioctl( skfd, IOCTL_CMD_SET, &sReq);
	
	if (result || sReq.wResult) {
		printf( "%s: failed in get bssid list command.\n", AppName);
		result = 1;
	}
	else {
	    if (sList.uItem == 0) return result;	    
		pList = malloc( sizeof(SBSSIDList) + (sList.uItem * sizeof(SBSSIDItem)));
		pList->uItem = sList.uItem;
		sReq.wResult = 0;
		sReq.data = pList;
		sReq.wCmdCode = WLAN_CMD_GET_LIST;
		result = ioctl( skfd, IOCTL_CMD_SET, &sReq);
		if (result || sReq.wResult) {
			printf( "%s: failed get bssid list command\n", AppName);
			result = 1;
		}
		else {
			vShowList(pList);
		}
		free(pList);
	}

	return result;
}


/*+
 *
 * Description:
 *
 *
 *	Show basic service sets list
 *
 * Parameters:
 *  In:
 *      pList
 *  Out:
 *      none
 *
 * Return Value: none
 *
-*/

VOID vShowList(PSBSSIDList pList)
{
	int 			    ii;
	BYTE 			    buf[80];

	    printf("CH  BSSID              BCN   TYPE  WEP RSSI  SSID\n");
	for (ii = 0; ii < pList->uItem; ii++) {
//	    printf("CH  BSSID              BCN   TYPE  WEP RSSI  SSID\n");
		printf("%03d  ", (int)pList->sBSSIDList[ii].uChannel);
		vAddrToStr(pList->sBSSIDList[ii].abyBSSID, buf);
		printf("%s  ", buf);
		printf("%04d  ", pList->sBSSIDList[ii].wBeaconInterval);
		if(pList->sBSSIDList[ii].byNetType == ADHOC) {
		    printf("Adhoc ");
		}
		else {
		    printf("Infra ");
        };
		if(pList->sBSSIDList[ii].bWEPOn == TRUE) {
		    printf("yes ");
		}
		else {
		    printf("no  ");
        };        
		printf("%04d  ", (int)pList->sBSSIDList[ii].uRSSI);
		memcpy(buf, pList->sBSSIDList[ii].abySSID, SSID_MAXLEN +1);
		printf("%s\n", buf);
	}
}

/*+
 *
 * Description:
 *
 *
 *	Called when the user has requested the 'nodes' command at the
 *	command line. 
 *
 * Parameters:
 *  In:
 *      skfd, argc, argv
 *  Out:
 *      none
 *
 * Return Value: '0' if success
 *
-*/


int s_cmdGetNodeList(int skfd, int argc, char **argv)
{
	int					result;
	SCmdRequest	        sReq;
	SNodeList		    sList;
    PSNodeList          pList;

	result = 0;
	memset(&sList, 0, sizeof(SNodeList));	
	strcpy((char *)sReq.name, argv[1]);
	sReq.wResult = 0;
	sReq.data = &sList;
	sReq.wCmdCode = WLAN_CMD_GET_NODE_CNT;
	result = ioctl( skfd, IOCTL_CMD_SET, &sReq);

	if (result || sReq.wResult) {
		printf( "%s: failed in get node list command.\n", AppName);
		result = 1;
	}
	else {
	    if (sList.uItem == 0) return result;
		pList = malloc( sizeof(SNodeList) + (sList.uItem * sizeof(SNodeItem)));
		pList->uItem = sList.uItem;
		sReq.wResult = 0;
		sReq.data = pList;
		sReq.wCmdCode = WLAN_CMD_GET_NODE_LIST;
		result = ioctl( skfd, IOCTL_CMD_SET, &sReq);
		if (result || sReq.wResult) {
			printf( "%s: failed get node list command\n", AppName);
			result = 1;
		}
		else {
			vShowNodeList(pList);
		}
		free(pList);
	}


	return result;
}


/*+
 *
 * Description:
 *
 *
 *	Show stations list
 *
 * Parameters:
 *  In:
 *      pList
 *  Out:
 *      none
 *
 * Return Value: none
 *
-*/

VOID vShowNodeList(PSNodeList pList)
{
	int 			    ii;
	BYTE 			    buf[80];
	
    printf("Associated Nodes Count = %d\n", (int)(pList->uItem -1));	
    printf("AID MAC                RATE AGE  QUE FG  PW  RATO  KEY LEN\n");
	for (ii = 0; ii < pList->uItem; ii++) {
		printf("%02d  ", pList->sNodeList[ii].wAID);
		vAddrToStr(pList->sNodeList[ii].abyMACAddr, buf);
		printf("%s  ", buf);
		printf("%02d  ", pList->sNodeList[ii].wTxDataRate);
		printf("%04d  ", pList->sNodeList[ii].wInActiveCount);
		printf("%02d  ", pList->sNodeList[ii].wEnQueueCnt);
		printf("%02x  ", pList->sNodeList[ii].wFlags);
		if(pList->sNodeList[ii].bPWBitOn == TRUE) {
		    printf("yes ");
		}
		else {
		    printf("no  ");
        };
		printf("%04d  ", pList->sNodeList[ii].wFailureRatio);
		printf("%02d  ", pList->sNodeList[ii].byKeyIndex);
		printf("%04d  ", pList->sNodeList[ii].wWepKeyLength);
		printf("\n");
		vWEPToStr(pList->sNodeList[ii].abyWepKey, buf);		
		printf("KEY = %s \n", buf);
	}
}

/*+
 *
 * Description:
 *
 *  Called when the user set the 'link' command at the
 *	command line to get link status
 *
 * Parameters:
 *  In:
 *      skfd, argc, argv
 *  Out:
 *      none
 *
 * Return Value: '0' if success
 *
-*/

int s_cmdGetLink(int skfd, int argc, char **argv)
{
	int					result;
	SCmdRequest	        sReq;
	SCmdLinkStatus	    sLink;
	BYTE 			    buf[SSID_MAXLEN + 1];


	result = 0;
	strcpy((char *)sReq.name, argv[1]);
	sReq.wResult = 0;
	sReq.data = &sLink;
	sReq.wCmdCode = WLAN_CMD_GET_LINK;
	result = ioctl(skfd, IOCTL_CMD_SET, &sReq);
	if (sReq.wResult ) {
		printf( "%s: failed in link command.\n", AppName);
	}
	else {
        if (sLink.bLink == TRUE) {
		    memcpy(buf, sLink.abySSID, SSID_MAXLEN + 1);
            if (sLink.wBSSType == ADHOC) {
                if (sLink.byState == ADHOC_STARTED)
                    printf("Adhoc Starting, SSID = '%s'\n", buf);
                else if (sLink.byState == ADHOC_JOINTED)
                    printf("Adhoc Link with, SSID = '%s'\n", buf);
            }
            else {
                if (sLink.wBSSType == INFRA)
                    printf("Link with AP, SSID = '%s'\n", buf);
            }  
            printf("Channel = %02d", (int)sLink.uChannel);
            vAddrToStr(sLink.abyBSSID, buf);
            printf(", Data Rate = ");
            switch(sLink.uLinkRate) {
                case RATE_1M:
                    printf("1Mbps"); 
                    break;
                case RATE_2M:
                    printf("2Mbps");
                    break;
                case RATE_5M:
                    printf("5.5Mbps");
                    break;
                case RATE_11M:
                    printf("11Mbps");
                    break;
                case RATE_6M:
                    printf("6Mbps");
                    break;          
                case RATE_9M:
                    printf("9Mbps");
                    break;
                case RATE_12M:
                    printf("12Mbps");
                    break;    
                case RATE_18M:
                    printf("18Mbps");
                    break;
                case RATE_24M:
                    printf("24Mbps");
                    break;
                case RATE_36M:
                    printf("36Mbps");
                    break;
                case RATE_48M:
                    printf("48Mbps");
                    break;
                case RATE_54M:
                    printf("54Mbps");
                    break;   
                default:
                    printf("54Mbps");                                      
            }
            printf("\n");            
            printf("BSSID = %s\n", buf);

        } 
        else {
            printf("Link fail, disconnected !\n");
        }                   
    }

	return result;
}


/*+
 *
 * Description:
 *
 *	Called when the user has requested the 'mib' command at the
 *	command line. 
 *
 * Parameters:
 *  In:
 *      skfd, argc, argv
 *  Out:
 *      none
 *
 * Return Value: '0' if success
 *
-*/

int s_cmdGetMIB(int skfd, int argc, char **argv)
{
	int					result;
	SCmdRequest	        sReq;
	SDot11MIBCount	    sMIB;


	result = 0;
	strcpy((char *)sReq.name, argv[1]);
	sReq.wResult = 0;
	sReq.data = &sMIB;
	sReq.wCmdCode = WLAN_CMD_GET_MIB;
	
	result = ioctl( skfd, IOCTL_CMD_SET, &sReq);
	
	if (result || sReq.wResult) {
		printf( "%s: failed in get mib command.\n", AppName);
		result = 1;
	}
	else {
  	    printf("TxFragmentCount =%12d  ", (INT)sMIB.TransmittedFragmentCount);
  	    printf("MulticastTxCount=%12d\n", (INT)sMIB.MulticastTransmittedFrameCount);
  	    printf("FailedCount     =%12d  ", (INT)sMIB.FailedCount);
  	    printf("RetryCount      =%12d\n", (INT)sMIB.RetryCount);
  	    printf("MultipleRetry   =%12d  ", (INT)sMIB.MultipleRetryCount);
  	    printf("RTSSuccessCount =%12d\n", (INT)sMIB.RTSSuccessCount);
  	    printf("RTSFailureCount =%12d  ", (INT)sMIB.RTSFailureCount);
  	    printf("ACKFailureCount =%12d\n", (INT)sMIB.ACKFailureCount);
  	    printf("DuplicateCount  =%12d  ", (INT)sMIB.FrameDuplicateCount);
  	    printf("RxFragmentCount =%12d\n", (INT)sMIB.ReceivedFragmentCount);
  	    printf("MulticastRxCount=%12d  ", (INT)sMIB.MulticastReceivedFrameCount);
  	    printf("FCSErrorCount   =%12d\n",(INT)sMIB.FCSErrorCount);
    }

	return result;
}



/*+
 *
 * Description:
 *
 *	Called when the user has requested the 'stat'(statistics)command at the
 *	command line. 
 *
 * Parameters:
 *  In:
 *      skfd, argc, argv
 *  Out:
 *      none
 *
 * Return Value: '0' if success
 *
-*/


int s_cmdGetSTAT(int skfd, int argc, char **argv)
{
	int					result;
	SCmdRequest	        sReq;
	SStatMIBCount	    sSTAT;


	result = 0;
	strcpy((char *)sReq.name, argv[1]);
	sReq.wResult = 0;
	sReq.data = &sSTAT;
	sReq.wCmdCode = WLAN_CMD_GET_STAT;
	result = ioctl( skfd, IOCTL_CMD_SET, &sReq);
	if (result || sReq.wResult) {
		printf("%s: failed in get stat command.\n", AppName);
		result = 1;
	}
	else {
  	    printf("IsrTx0OK      =%12d  ", (INT)sSTAT.dwIsrTx0OK);
  	    printf("IsrTx1OK      =%12d\n", (INT)sSTAT.dwIsrTx1OK);
  	    printf("IsrBeaconTxOK =%12d  ", (INT)sSTAT.dwIsrBeaconTxOK);
  	    printf("IsrRxOK       =%12d\n", (INT)sSTAT.dwIsrRxOK);
  	    printf("IsrTBTTInt    =%12d  ", (INT)sSTAT.dwIsrTBTTInt);
  	    printf("IsrSTIMERInt  =%12d\n", (INT)sSTAT.dwIsrSTIMERInt);
  	    printf("IsrUnrecovErr =%12d  ", (INT)sSTAT.dwIsrUnrecoverableError);
  	    printf("IsrSoftInt    =%12d\n", (INT)sSTAT.dwIsrSoftInterrupt);
  	    printf("IsrRxNoBuf    =%12d  ", (INT)sSTAT.dwIsrRxNoBuf);
  	    printf("IsrUnknown    =%12d\n", (INT)sSTAT.dwIsrUnknown);

  	    printf("RsrFrmAlgnErr =%12d  ", (INT)sSTAT.dwRsrFrmAlgnErr);
  	    printf("RsrErr        =%12d\n", (INT)sSTAT.dwRsrErr);
  	    printf("RsrCRCErr     =%12d  ", (INT)sSTAT.dwRsrCRCErr);
  	    printf("RsrCRCOk      =%12d\n", (INT)sSTAT.dwRsrCRCOk);  
  	    printf("RsrBSSIDOk    =%12d  ", (INT)sSTAT.dwRsrBSSIDOk);
  	    printf("RsrADDROk     =%12d\n", (INT)sSTAT.dwRsrADDROk);  
  	    printf("RsrICVOk      =%12d  ", (INT)sSTAT.dwRsrICVOk);
  	    printf("RsrShortPre   =%12d\n", (INT)sSTAT.dwNewRsrShortPreamble);
  	    printf("RsrLong       =%12d  ", (INT)sSTAT.dwRsrLong);
  	    printf("RsrRunt       =%12d\n", (INT)sSTAT.dwRsrRunt);
  	    printf("RsrRxControl  =%12d  ", (INT)sSTAT.dwRsrRxControl);
  	    printf("RsrRxData     =%12d\n", (INT)sSTAT.dwRsrRxData);
  	    printf("RsrRxManage   =%12d  ", (INT)sSTAT.dwRsrRxManage);
  	    printf("RsrRxPacket   =%12d\n", (INT)sSTAT.dwRsrRxPacket);
  	    printf("RsrRxOctet    =%12d  ", (INT)sSTAT.dwRsrRxOctet);
  	    printf("RsrBroadcast  =%12d\n", (INT)sSTAT.dwRsrBroadcast);
  	    printf("RsrMulticast  =%12d  ", (INT)sSTAT.dwRsrMulticast);
  	    printf("RsrDirected   =%12d\n", (INT)sSTAT.dwRsrDirected);
  	    printf("RsrOK         =%12d\n\n", (INT)sSTAT.ullRsrOK);
  	    
  	    printf("RxBroadcstBytes=%12d  ", (INT)sSTAT.ullRxBroadcastBytes);
  	    printf("RxMulticstBytes=%12d\n", (INT)sSTAT.ullRxMulticastBytes);
  	    printf("RxDirectedBytes=%12d  ", (INT)sSTAT.ullRxDirectedBytes);
  	    printf("RxBroadcstFrame=%12d\n", (INT)sSTAT.ullRxBroadcastFrames);
  	    printf("RxMulticstFrame=%12d  ", (INT)sSTAT.ullRxMulticastFrames);
  	    printf("RxDirectedFrame=%12d\n", (INT)sSTAT.ullRxDirectedFrames);
  	    

  	    printf("RxLen64        =%12d  ", (INT)sSTAT.dwRsrRxFrmLen64);
  	    printf("RxLen65_127    =%12d\n", (INT)sSTAT.dwRsrRxFrmLen65_127);
  	    printf("RxLen128_255   =%12d  ", (INT)sSTAT.dwRsrRxFrmLen128_255);
  	    printf("RxLen256_511   =%12d\n", (INT)sSTAT.dwRsrRxFrmLen256_511);
  	    printf("RxLen512_1023  =%12d  ", (INT)sSTAT.dwRsrRxFrmLen512_1023);
  	    printf("RxLen1024_1518 =%12d\n", (INT)sSTAT.dwRsrRxFrmLen1024_1518);
  	    printf("RsrRxOctet     =%12d  ", (INT)sSTAT.dwRsrRxOctet);
  	    printf("RsrBroadcast   =%12d\n", (INT)sSTAT.dwRsrBroadcast);
  	    printf("RsrMulticast   =%12d  ", (INT)sSTAT.dwRsrMulticast);
  	    printf("RsrDirected    =%12d\n", (INT)sSTAT.dwRsrDirected);
  	    printf("RsrOK          =%12d  ", (INT)sSTAT.ullRsrOK);
  	    printf("RxBroadcstByte =%12d\n", (INT)sSTAT.ullRxBroadcastBytes);
  	    printf("RsrRxFragment  =%12d\n\n", (INT)sSTAT.dwRsrRxFragment);  	    

  	    printf("TsrTotalRetry  =%12d  ", (INT)sSTAT.dwTsrTotalRetry[1]);
  	    printf("TsrOnceRetry   =%12d\n", (INT)sSTAT.dwTsrOnceRetry[1]);
  	    printf("TsrMoreOneRetry=%12d  ", (INT)sSTAT.dwTsrMoreThanOnceRetry[1]);
  	    printf("TsrRetry       =%12d\n", (INT)sSTAT.dwTsrRetry[1]);
  	    printf("TsrACKData     =%12d  ", (INT)sSTAT.dwTsrACKData[1]);
  	    printf("TsrErr         =%12d\n", (INT)sSTAT.dwTsrErr[1]);
  	    printf("AllTsrOK       =%12d  ", (INT)sSTAT.dwAllTsrOK[1]);
  	    printf("TsrRetryTimeout=%12d\n", (INT)sSTAT.dwTsrRetryTimeout[1]);
  	    printf("TsrTxTimeout   =%12d  ", (INT)sSTAT.dwTsrTransmitTimeout[1]);
  	    printf("TsrTxPacket    =%12d\n", (INT)sSTAT.dwTsrTxPacket[1]);
  	    printf("TsrTxOctet     =%12d  ", (INT)sSTAT.dwTsrTxOctet[1]);
  	    printf("TsrBroadcast   =%12d\n", (INT)sSTAT.dwTsrBroadcast[1]);
  	    printf("TsrMulticast   =%12d  ", (INT)sSTAT.dwTsrMulticast[1]);
  	    printf("TsrDirected    =%12d\n", (INT)sSTAT.dwTsrDirected[1]);
/*  	    
  	    printf("CntRxDataErr   =%12d  ", (INT)sSTAT.dwCntRxDataErr);
  	    printf("CntDecryptErr  =%12d\n", (INT)sSTAT.dwCntDecryptErr);
  	    printf("CntRxICVErr    =%12d  ", (INT)sSTAT.dwCntRxICVErr);
  	    printf("RxErrorDesc    =%12d\n", (INT)sSTAT.idxRxErrorDesc);
*/  	    
  	    
    }            

	return result;
}

/*+
 *
 * Description:
 *
 *  This function displays usages of the wetctl utility.
 *
 *
 * Parameters:
 *  In:
 *      void
 *  Out:
 *      none
 *
 * Return Value: none
 *
-*/

VOID vCmdUsage(void)
{
	printf("\nUtility for Solomon-A/B/G wireless cards: %s Version %s\n", AppName, VERSION);
	printf("Usage: %s interface command\n\n", AppName);
	printf("   where \"interface\" is the name of a network interface\n");
	printf("   such as eth0 or eth1.. Running 'ifconfig' will list all\n");
	printf("   network interfaces.\n\n");
	printf("Example: %s eth0 link\n\n", AppName);	
	printf("Command List:               \n");	
	printf("   scan : scan all channels to collect available BSS(AP)\n");
	printf("   list : show all BSS(AP) list\n");
	printf("   link : show link status\n");	
	printf("   mib  : show 802.11 mib counters\n");
//	printf("   stop : turn off radio\n");
//	printf("   stat : show statistics\n");
//	printf("   nodes: show station nodes\n");
}
